use Test::More tests => 1;

BEGIN {
    use_ok('CGI::Application::Plugin::Cache::Adaptive');
}

diag( "Testing CGI::Application::Plugin::Cache::Adaptive $CGI::Application::Plugin::Cache::Adaptive::VERSION" );
